local class = require 'middleclass'
local Stateful = require 'Stateful'

Game = class('Game')
Game:include(Stateful)

function Game:initialize(state)
	self:gotoState(state)
end

Menu = Game:addState('Menu')

function Menu:enteredState()
	love.graphics.setBackgroundColor(248, 138, 175)
	button1 = Button:new(212, 240, 45, 16, 'PLAY', function() self:gotoState('Play') end)
	button2 = Button:new(196, 265, 80, 16, 'CONTROLS', function() self:gotoState('Controls') end)

end

function Menu:update(dt)
	button1:update(dt)
	button2:update(dt)
end

function Menu:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(titleImg, 90, 80)
	love.graphics.setFont(smallFont)
	button1:draw()
	button2:draw()
end

function Menu:exitedState()
	--button1 = nil
	--button2 = nil
end

Controls = Game:addState('Controls')

function Controls:enteredState()
	button = Button:new(230, 320, 40, 20, 'BACK', function() self:gotoState('Menu') end)
end

function Controls:update(dt)
	button:update(dt)
	if love.keyboard.isDown('escape') then
		self:gotoState('Menu')
	end
end

function Controls:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(controlsImg, 0, 0)
	button:draw()
end

function Controls:exitedState()
	button = nil
end

Play = Game:addState('Play')

function Play:enteredState()
	cell = {}
	cell.x = 0
	cell.y = 0
	cell.size = 55
	cell.hp = 50

	cameraOffsetX = 0
	cameraShaking = false
	cameraShakeAmplitude = 0
	cameraShakeTime = 0
	cameraShakeLength = 0

	player = Player:new()

	playerBullets = Container:new()
	enemies = Container:new()
	enemyBullets = Container:new()

	love.mouse.setVisible(false)
	love.mouse.setPosition(love.window.getWidth()/2, love.window.getHeight()/2)

	love.graphics.setBackgroundColor(248, 138, 175)
	score = 0

	architect = Architect:new()
	local randomAngle = math.random() * 2 * math.pi
	enemies:add(Tiny:new(320*math.cos(randomAngle), 320*math.sin(randomAngle)))

	cameraShaking = true
	cameraShakeLength = 1
	cameraShakeAmplitude = 4
	startSound:play()
end

function Play:update(dt)
	architect:update(dt)
	player:update(dt)
	playerBullets:update(dt)
	enemies:update(dt)
	enemyBullets:update(dt)

	if player.dmgTimeSinceLastShot > player.dmgReloadTime and player.sloTimeSinceLastShot > player.sloReloadTime then
		cell.hp = cell.hp + 2.5 * dt
	end

	if cell.hp >= 50 then
		cell.hp = 50
		score = score + 8 * dt
	end

	if cell.hp > 0 then
		score = score + 2 * dt
	end

	if cell.hp < -50 then
		fail:rewind()
		fail:play()
		self:gotoState('GameOver')
	end

	if love.keyboard.isDown('k') and math.random() > 0.8 then enemies:add(Bomber:new(300, 300)) end
	if love.keyboard.isDown('l') and math.random() > 0.8 then enemies:add(Sapper:new(-300, -300)) end

	if love.keyboard.isDown('escape') then
		self:gotoState('Menu')
	end

	if cameraShaking then
		cameraShakeTime = cameraShakeTime + dt
		cameraOffsetX = cameraShakeAmplitude * math.cos(cameraShakeTime * 10 * 2 * math.pi) * (1 - cameraShakeTime/cameraShakeLength)

		if cameraShakeTime > cameraShakeLength then
			cameraShaking = false
			cameraShakeAmplitude = 0
			cameraShakeTime = 0
			cameraShakeLength = 0
		end
	end
	orbiterAnim:update(dt)
end

function Play:draw()
	love.graphics.push()
	love.graphics.translate(240 + cameraOffsetX, 180)
	love.graphics.setLineWidth(2)

	love.graphics.setColor(255, 255, 255)
	love.graphics.circle('line', cell.x, cell.y, cell.size)
	if cell.hp > 0 then
		love.graphics.circle('fill', cell.x, cell.y, cell.hp/50 * intRound(cell.size))
	else
		love.graphics.setColor(30, 38, 37)
		love.graphics.circle('fill', cell.x, cell.y, -cell.hp/50 * intRound(cell.size))
	end
	enemies:draw()
	playerBullets:draw()
	enemyBullets:draw()
	player:draw()
	love.graphics.pop()
	love.graphics.setColor(255, 255, 255)
	love.graphics.printf(math.floor(score), 473, 10, 0, 'right')

end

function Play:exitedState()
	player = nil
	cell = nil
	ayerBullets = nil
	enemies = nil
	enemyBullets = nil
	love.mouse.setVisible(true)
end

GameOver = Game:addState('GameOver')

function GameOver:enteredState()
	playAgainButton = Button:new(200, 260, 100, 20, 'PLAY AGAIN', function() self:gotoState('Play') end)
	menuButton = Button:new(230, 310, 50, 20, 'MENU', function() self:gotoState('Menu') end)
end

function GameOver:update(dt)
	playAgainButton:update(dt)
	menuButton:update(dt)
end

function GameOver:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.setFont(smallFont)
	love.graphics.print('SCORE', 222, 80)
	love.graphics.setFont(hugeFont)
	love.graphics.printf(math.floor(score), 250, 120, 0, 'center')
	playAgainButton:draw()
	menuButton:draw()
end

function GameOver:exitedState()
end