local class = require 'middleclass'

Orbiter = class('Orbiter')

function Orbiter:initialize(x, y)
	self.x = x
	self.y = y

	self.size = 9
	self.speed = 60 + math.random(40)
	local dx = self.x - cell.x
	local dy = self.y - cell.y
	local a = math.atan2(dy, dx)
	local ta = normalizeAngle(a + math.pi/2)
	self.orbitHeight = 120 + math.random() * 30
	self.tx = cell.x + math.cos(ta) * self.orbitHeight
	self.ty = cell.y + math.sin(ta) * self.orbitHeight

	local dx = self.tx - self.x
	local dy = self.ty - self.y
	local d = math.sqrt(dx^2 + dy^2)
	self.vx = dx * self.speed / d
	self.vy = dy * self.speed / d

	self.hp = 40

	self.toDelete = false
	self.orbiting = false
	self.ca = 0
	self.av = self.speed / self.orbitHeight

	self.anim = orbiterAnim

	self.timeSinceLastShot = 0
	self.reloadTime = 0.5
end

function Orbiter:update(dt)
	if not self.orbiting then
		local dx = self.tx - self.x
		local dy = self.ty - self.y
		local d = math.sqrt(dx^2 + dy^2)
		self.vx = dx * self.speed / d
		self.vy = dy * self.speed / d
		self.x = self.x + self.vx * dt
		self.y = self.y + self.vy * dt

		if math.sqrt((self.x - self.tx)^2 + (self.y - self.ty)^2) < 10 then
			self.orbiting = true
			self.ca = math.atan2(self.y - cell.y, self.x - cell.x)
			self.orbitHeight = math.sqrt((self.y - cell.y)^2 + (self.x - cell.x)^2)
		end
	else
		self.av = self.speed / self.orbitHeight
		self.ca = normalizeAngle(self.ca + self.av * dt)
		self.x = cell.x + math.cos(self.ca) * self.orbitHeight
		self.y = cell.y + math.sin(self.ca) * self.orbitHeight
		self.timeSinceLastShot = self.timeSinceLastShot + dt
		if math.random() > 0.98 and self.timeSinceLastShot > self.reloadTime then
			self.timeSinceLastShot = 0
			local a = math.atan2(cell.y - self.y, cell.x - self.x)
			enemyBullets:add(EnemyBullet:new(self.x, self.y, 250 * math.cos(a), 250 * math.sin(a), 3, 0, 5))
			shoot3:rewind()
			shoot3:play()
		end
	end

	if self.hp <= 0 then
		self.toDelete = true
		dmg:rewind()
		dmg:play()
		score = score + 150
	end
end

function Orbiter:draw()
	love.graphics.setColor(30, 38, 37)
	--love.graphics.draw(orbiterImg, self.x, self.y, math.atan2(self.y, self.x) + math.pi/2, 1, 1, 32, 16)
	self.anim:draw(self.x, self.y, math.atan2(self.y, self.x) + math.pi/2, 1, 1, 32, 16)
end