local class = require 'middleclass'
local Stateful = require 'Stateful'

Player = class('Player')
Player:include(Stateful)

function Player:initialize()
	self.x = 0
	self.y = cell.y - cell.size - 12

	self.aimOffsetX = 0
	self.aimOffsetY = 0
	self.aimAngle = 0
	self.aimMaxOffset = 100
	self.aimSensitivity = 0.4

	self.speed = 200
	self.size = 12

	self.dmgTimeSinceLastShot = -0.4
	self.dmgReloadTime = 0.09

	self.sloTimeSinceLastShot = 0.4
	self.sloReloadTime = 1.4

	self.direction = 1
	self.moving = false

	self.rv = 0
	self.cellAngle = math.atan2(self.y - cell.y, self.x - cell.x)

	crosshairPositions = {}

	self.shooting = false
end

function Player:update(dt)
	-- keyboard movement
	self.cellAngle = math.atan2(self.y - cell.y, self.x - cell.x)
	local da = 0

	if love.keyboard.isDown('d', 'e', 'right') then
		self.rv = self.rv + self.speed / cell.size * 0.7
		self.moving = true
		self.direction = 1
	elseif love.keyboard.isDown('a', 'q', 'left') then
		self.rv = self.rv - self.speed / cell.size * 0.7
		self.moving = true
		self.direction = -1
	else
		self.moving = false
	end

	if self.rv > self.speed/cell.size then
		self.rv = self.speed/cell.size
	elseif self.rv < -self.speed/cell.size then
		self.rv = -self.speed/cell.size
	end

	da = self.rv * dt
	self.cellAngle = normalizeAngle(self.cellAngle + da)
	self.x = cell.x + (cell.size + self.size) * math.cos(self.cellAngle)
	self.y = cell.y + (cell.size + self.size) * math.sin(self.cellAngle)
	--self.aimOffsetX = self.aimOffsetX * math.cos(da) - self.aimOffsetY * math.sin(da) -- rotate crosshair
	--self.aimOffsetY = self.aimOffsetX * math.sin(da) + self.aimOffsetY * math.cos(da)

	self.rv = self.rv * 0.45

	-- mouse aiming
	local dx = love.mouse.getX() - intRound(love.window.getWidth()/2)
	local dy = love.mouse.getY() - intRound(love.window.getHeight()/2)
	love.mouse.setPosition(intRound(love.window.getWidth()/2), intRound(love.window.getHeight()/2))

	self.aimOffsetX = self.aimOffsetX + dx * self.aimSensitivity
	self.aimOffsetY = self.aimOffsetY + dy * self.aimSensitivity

	local aimOffset = math.sqrt(self.aimOffsetX^2 + self.aimOffsetY^2)
	if aimOffset > self.aimMaxOffset then
		aimOffset = self.aimMaxOffset
	end

	self.aimAngle = math.atan2(self.aimOffsetY, self.aimOffsetX)
	local angleDiff = normalizeAngle(self.cellAngle - self.aimAngle)

	if angleDiff > math.pi/2 + 0.4 then
		self.aimAngle = self.cellAngle - (math.pi/2 + 0.4)
	elseif angleDiff < -(math.pi/2 + 0.4) then
		self.aimAngle = self.cellAngle + (math.pi/2 + 0.4)
	end

	self.aimOffsetX = aimOffset * math.cos(self.aimAngle)
	self.aimOffsetY = aimOffset * math.sin(self.aimAngle)

	local relativeAimAngle = normalizeAngle(self.cellAngle - self.aimAngle + math.pi/2)

	if self.moving then
		playerAnim:update(dt)
	else
		playerAnim:reset()
		if relativeAimAngle < math.pi/2 and relativeAimAngle > -math.pi/2 then
			self.direction = 1
		else
			self.direction = -1
		end
	end


	-- shooting
	self.dmgTimeSinceLastShot = self.dmgTimeSinceLastShot + dt
	self.sloTimeSinceLastShot = self.sloTimeSinceLastShot + dt


	self.shooting = false
	if love.mouse.isDown('l') then
		if self.dmgTimeSinceLastShot >= self.dmgReloadTime then
			self.dmgTimeSinceLastShot = 0
			local ao = math.random() * 0.2 - 0.1
			playerBullets:add(Bullet:new(self.x + 3*math.cos(self.aimAngle), self.y+3*math.sin(self.aimAngle), 550*math.cos(self.aimAngle + ao), 550*math.sin(self.aimAngle + ao), 8, 1, 5))
			shoot1:rewind()
			shoot1:play()
			if math.abs(math.pi/2 - relativeAimAngle) > math.pi/4 then 
				self.rv = self.rv - 0.2 * self.direction
			end
			self.shooting = true
		end
	elseif love.mouse.isDown('r') then
		if self.sloTimeSinceLastShot >= self.sloReloadTime then
			self.sloTimeSinceLastShot = 0
			local ao = math.random(100) / 1000 - 0.05
			playerBullets:add(Bullet:new(self.x + 16*math.cos(self.aimAngle), self.y+16*math.sin(self.aimAngle), 200*math.cos(self.aimAngle), 200*math.sin(self.aimAngle), 8, 0.4, 0))
			shoot2:rewind()
			shoot2:play()
		end
	end

end

function Player:draw()
	--love.graphics.circle('fill', self.x, self.y, self.size)
	love.graphics.setColor(255, 255, 255)
	if self.moving then
		playerAnim:draw(self.x, self.y, self.cellAngle + math.pi/2, self.direction, 1, playerImg:getWidth()/2, playerImg:getHeight() - 12)
	else
		love.graphics.draw(playerImg, self.x, self.y, self.cellAngle + math.pi/2, self.direction, 1, playerImg:getWidth()/2, playerImg:getHeight() - 12)
		local relativeAimAngle = normalizeAngle(self.cellAngle - self.aimAngle + math.pi/2)
		if self.direction == 1 then
			if relativeAimAngle > 0 then
				love.graphics.draw(playerHeadImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2 - relativeAimAngle/2.2, 1, 1, 16, 16)
				love.graphics.draw(gunImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2 - relativeAimAngle/2, 1, 1, 16, 16)
			else
				love.graphics.draw(playerHeadImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2, 1, 1, 16, 16)
				love.graphics.draw(gunImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2, 1, 1, 16, 16)
			end
		else
			if relativeAimAngle > 0 then
				love.graphics.draw(playerHeadImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2 + math.abs(math.pi/2-relativeAimAngle/2.2), -1, 1, 16, 16)
				love.graphics.draw(gunImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2 + math.abs(math.pi/2-relativeAimAngle/2), -1, 1, 16, 16)
			else
				love.graphics.draw(playerHeadImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2, -1, 1, 16, 16)
				love.graphics.draw(gunImg, (cell.size + self.size + 4)*math.cos(self.cellAngle), (cell.size + self.size + 4)*math.sin(self.cellAngle), self.cellAngle + math.pi/2, -1, 1, 16, 16)
			end
		end

	end
	table.insert(crosshairPositions, {x=self.x + self.aimOffsetX, y=self.y + self.aimOffsetY})
	if #crosshairPositions > 10 then
		table.remove(crosshairPositions, 1)
	end
	for i=1, #crosshairPositions do
		love.graphics.setColor(255, 255, 255, 255*i^2/100)
		love.graphics.circle('fill', crosshairPositions[i].x, crosshairPositions[i].y, 1)
	end

	if self.shooting then
		love.graphics.circle('fill', self.x + 10*math.cos(self.aimAngle), self.y+10*math.sin(self.aimAngle), 8)
	end
end